"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chalk_1 = tslib_1.__importDefault(require("chalk"));
const gray_matter_1 = tslib_1.__importDefault(require("gray-matter"));
const path_1 = tslib_1.__importDefault(require("path"));
const glob_1 = tslib_1.__importDefault(require("glob"));
const unified_1 = tslib_1.__importDefault(require("unified"));
const fs_extra_1 = tslib_1.__importDefault(require("fs-extra"));
const remark_parse_1 = tslib_1.__importDefault(require("remark-parse"));
const rehype_format_1 = tslib_1.__importDefault(require("rehype-format"));
const remark_rehype_1 = tslib_1.__importDefault(require("remark-rehype"));
const rehype_stringify_1 = tslib_1.__importDefault(require("rehype-stringify"));
const rehype_raw_1 = tslib_1.__importDefault(require("rehype-raw"));
const fs_extra_promise_1 = tslib_1.__importDefault(require("fs-extra-promise"));
const remark_frontmatter_1 = tslib_1.__importDefault(require("remark-frontmatter"));
const engine_1 = require("./engine");
const generator_1 = require("./generator");
const page_1 = require("./page");
const orderBy_1 = tslib_1.__importDefault(require("lodash/orderBy"));
const defaultConfig = {
    sourcePath: "src",
    publicPath: "public",
    basePath: "/",
    staticPath: "static",
    assetsPath: "assets"
};
exports.compiler = (options = defaultConfig) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const isTestRunner = process.env.NODE_ENV === "test";
    const startTime = process.hrtime();
    const basePath = options.basePath;
    const sourcePath = path_1.default.resolve(basePath, options.sourcePath);
    const publicPath = path_1.default.resolve(basePath, options.publicPath);
    const staticPath = path_1.default.resolve(sourcePath, options.staticPath);
    const staticDestinationPath = path_1.default.resolve(publicPath, options.staticPath);
    const pagesPath = `${sourcePath}/pages`;
    function writeFile(destination, content) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield fs_extra_1.default.ensureFile(destination);
            yield fs_extra_promise_1.default.writeFileAsync(destination, content, { encoding: "utf8" });
        });
    }
    // tslint:disable-next-line no-unused-expression
    !isTestRunner && console.log(chalk_1.default.green("Building site..."));
    const engine = new engine_1.HandlebarsEngine({
        partialsDir: `${sourcePath}/partials`,
        layoutsDir: `${sourcePath}/layouts`,
        helpersDir: `${sourcePath}/helpers`
    });
    const generator = new generator_1.Generator(engine);
    // clean output directory
    const deletionList = glob_1.default.sync(`!(${options.staticPath}|${options.assetsPath})**`, { cwd: publicPath });
    for (const d of deletionList) {
        const p = path_1.default.resolve(publicPath, d);
        const stats = fs_extra_1.default.statSync(p);
        if (stats.isFile()) {
            yield fs_extra_promise_1.default.removeAsync(p);
        }
        if (stats.isDirectory()) {
            yield fs_extra_promise_1.default.emptyDir(p);
        }
    }
    // Collect a list of files
    const files = glob_1.default.sync("**/*.@(md|markdown)", { cwd: pagesPath });
    const pages = [];
    for (const f of files) {
        // raw content of markdown source
        const fContent = yield fs_extra_promise_1.default.readFileAsync(`${pagesPath}/${f}`, {
            encoding: "utf8"
        });
        const result = yield unified_1.default()
            .use(remark_parse_1.default)
            .use(remark_frontmatter_1.default, ["yaml", "toml"])
            .use(remark_rehype_1.default, { allowDangerousHTML: true })
            .use(rehype_raw_1.default)
            .use(rehype_format_1.default)
            .use(rehype_stringify_1.default)
            .process(fContent);
        const { content } = gray_matter_1.default(String(result), {
            excerpt: true
        });
        const { data, excerpt } = gray_matter_1.default(String(fContent), {
            excerpt: true
        });
        const { dir, name } = path_1.default.parse(f);
        const page = new page_1.Page(content, data, excerpt, name, `${publicPath}${dir && "/" + dir}`);
        pages.push(page);
    }
    for (const p of pages) {
        // serialize all pages so that themes can loop over them
        const serializedPages = [
            ...orderBy_1.default(pages.map(s => s.serialize(publicPath, s.getDestinationPath() === p.getDestinationPath())), ["relativePath"], "desc")
        ];
        // generate rendered html result
        const htmlOutput = yield generator.render({
            body: p.htmlContent(),
            meta: p.getMeta(),
            title: p.getName(),
            slug: p.getSlug(),
            excerpt: p.getExcerpt(),
            pages: serializedPages
        });
        // write result to disk
        yield writeFile(p.getDestinationPath(), htmlOutput);
    }
    // copy static asset folder
    yield fs_extra_promise_1.default.copyAsync(staticPath, staticDestinationPath, {
        overwrite: true,
        preserveTimestamps: true
    });
    // display build time
    const timeDiff = process.hrtime(startTime);
    const duration = timeDiff[0] * 1000 + timeDiff[1] / 1e6;
    // tslint:disable-next-line no-unused-expression
    !isTestRunner &&
        console.log(chalk_1.default.green(`Site built successfully in ${duration}ms`));
    return;
});
